#!/bin/bash
# © Copyright [2016 - 2020] Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates and licensors
# (“Micro Focus”) are as may be set forth in the express warranty statements accompanying such
# products and services. Nothing herein should be construed as constituting an additional
# warranty. Micro Focus shall not be liable for technical or editorial errors or omissions
# contained herein. The information contained herein is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains confidential information
# and a valid license is required for possession, use or copying. If this work is provided to the
# U.S. Government, consistent with FAR 12.211 and 12.212, Commercial Computer Software,
# Computer Software Documentation, and Technical Data for Commercial Items are licensed
# to the U.S. Government under vendor's standard commercial license.

# Start Consul as a server
nohup /consul/consul agent -server -config-dir=/etc/consul.d -data-dir=/tmp/consul -node=cis-gateway -bootstrap-expect=1 -dns-port=53 -log-level=warn 2>/dev/null &
status=$?
if [ $status -ne 0 ]; then
  echo "Failed to start Consul server: $status"
  exit $status
fi

# Start the Envoy process
nohup /envoy/envoy -c /conf/front-envoy.json --service-cluster oes_cis --service-node api_gateway &
status=$?
if [ $status -ne 0 ]; then
  echo "Failed to start Front Envoy: $status"
  exit $status
fi

# Start Cluster Discovery wrapper binary
nohup /cdsServer &
status=$?
if [ $status -ne 0 ]; then
  echo "Failed to start CDS server: $status"
  exit $status
fi

trap terminate TERM
terminate()
{
    echo "Signal received, stopping..."
    echo "Main process exited gracefully."
    /consul/consul leave
    echo "Gracefully exited Consul agent."
    exit 0
}

# Health check for the services that are invoked above.
# The container exits with an error if any one of the three processes has exited.
# Otherwise it loops forever, waking up every 60 seconds.

while true; do
  sleep 60&
  wait $!
  ps aux |grep envoy/envoy |grep -q -v grep
  ENVOY_STATUS=$?
  ps aux |grep consul/consul |grep -q -v grep
  CONSUL_STATUS=$?
  ps aux |grep cdsServer |grep -q -v grep
  CDS_STATUS=$?
  
  # Exit the Script with PID 1 in container, which in turn causes the container to go down 
  if [ $ENVOY_STATUS -ne 0 -o $CONSUL_STATUS -ne 0 -o $CDS_STATUS -ne 0 ]; then
    echo "One of the processes has already exited."
    echo "ENVOY_STATUS"
    echo $ENVOY_STATUS
    echo "CONSUL_STATUS"
    echo $CONSUL_STATUS
    echo "CDS_STATUS"
    echo $CDS_STATUS
    /consul/consul leave 2>/dev/null
    exit 1
  fi
done